<?xml version="1.0"?>
<!DOCTYPE methoddef SYSTEM "rpc-method.dtd">
<!--
    Generated automatically by make_method v1.09, Sun Sep 12 15:24:02 2004

    Any changes made here will be lost.
-->
<methoddef>
<name>sync</name>
<version>1.0</version>
<signature>array array</signature>
<help>
Syncs configuration files in /etc/asterisk.
</help>
<code language="perl">
<![CDATA[
#!/opt/perl5/bin/perl
###############################################################################
#
#   Sub Name:       sync_config
#
#   Description:    Syncs configuraiton files on the client machine
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $srv      in      ref       Server object instance
#
#   Globals:        None.
#
#   Environment:    None.
#
#   Returns:        None.
#
###############################################################################
sub sync_config
{
        my ($srv, $req) = @_;
open(LOG, ">>/tmp/xpl.log"); print LOG localtime() . ": $$: sync.xpl(@_)\n"; close(LOG);

	use strict;
	use File::Copy;
	use Fcntl ':flock';
	use Digest::MD5 qw/md5_hex/;


	my ($file, $md5, $data, $path, $noreload,$mosted_ok) = @{$req};
	# Check for mosted support - we pass an additional dummy parameter to verify that the 
	# call is from a modern RPC lib.
	if ( -e "/etc/asterisk/unbound" && !$mosted_ok )
	{
		return "This is an UNBOUND server.  You are attempting to access this server using code that does not support Unbound servers";
	}

	my $origfile = $file;
	unless ($file)
	{
		return "No file specified";
	}
	unless ($md5)
	{
		return "No md5 specified";
	}
	$file =~ s|^/etc/asterisk/||;

	# allowed path(s)
	if ($path && $path !~ m;^(/data/\d*/*)?(/usr/local/fonality|/usr/local/ircd/etc|/tftpboot|/etc/asterisk|/var/lib/asterisk|/etc/postfix|/usr/local/tftpd(?:/|$)|/var/spool/asterisk/outgoing(?:/|$));)
	{
		return "Cannot allow you to write to that directory";
	}

	if ( !$path && $file =~ m;^(/data/\d+/)(.*); )
	{
		$path = $1;
		$file = $2;
	} 
	

		
	$path ||= "/etc/asterisk";


	if ($file =~ /(?:tftp|smg_bri|wanpipe)/)
	{
		$path = "";	
	}

	if (length($data))
	{
		# check if the md5 sum of the file matches the md5 of the file they want to sync
		if (open(FILE, "<$path/$file"))
		{
			if (md5_hex(<FILE>) eq $md5)
			{
				close(FILE);
				return 1;
			}
			close(FILE);
		}

		# make directories for any directories that are not made
		my $temp;

		# go through each directory and create it if it hasn't already been
		foreach ((split(m|/|, "$path/$file"))[0 .. "$path/$file" =~ tr|/|| - 1])
		{
			$temp .= "$_/";
			if (-e $temp)
			{
				unless (-d $temp)
				{
					return "$temp is not a directory";
				}
			}
			else
			{
				mkdir($temp) || return "Can't mkdir($temp): $!";
			}
		}


		# use a semaphore in case the this process and another
		# do the open() before either perform a flock()
		my $semaphore = "$path/$file.lck";
		open(SEMAPHORE, ">$semaphore") || die "Can't open semaphore $semaphore for writing: $!";

		flock(SEMAPHORE, LOCK_EX);

		# backup the file we're going to overwrite
		copy("$path/$file", "$path/bak/$file." . time() . ".$$");

		open(FILE, ">$path/$file") || return "Unable to write to $path/$file: $!";
		print FILE $data;
		close FILE;


		close(SEMAPHORE);
		unlink($semaphore);

		if (!$noreload && $origfile =~ m|^(/data/\d+)?/etc/asterisk/|)
		{
			if ($origfile =~ m|/sip\.conf$|)
			{
				system("/var/www/rpc/astreload", "chan_sip.so");
			}
			elsif ($origfile =~ m|/musiconhold\.conf$|)
			{
				system("/var/www/rpc/astreload", "res_musiconhold.so");
			}
			elsif ($origfile =~ m|/iax\.conf$|)
			{
				system("/var/www/rpc/astreload", "chan_iax2.so");
			}
			elsif ($origfile =~ m^/fonality/|/extensions\.conf$^)
			{
				system("/var/www/rpc/astreload", "pbx_config.so");
			}
			elsif ($origfile =~ m^/parking\.conf$|/features\.conf$^)
			{
				# there is no res_features in 1.6, just do standard reload
				system("/var/www/rpc/astreload");
			}
			elsif ($origfile =~ m|/queues\.conf$|)
			{
				system("/var/www/rpc/astreload", "app_queue.so");
			}
			elsif ($origfile =~ m|/voicemail\.conf$|)
			{
				system("/var/www/rpc/astreload", "app_voicemail.so");
			}
			elsif ($origfile =~ m|/meetme\.conf$|)
			{
				system("/var/www/rpc/astreload", "app_meetme.so");
			}
			elsif ($origfile =~ m^/(?:alerts|agents|astwatch|pbxtra)\.conf$|\.xml$^)
			{
				# ignore 
			}
			elsif ($origfile =~ /\.conf$/)
			{
				system("/var/www/rpc/astreload");
			}
		}


		return 1;
	}

	else
	{
		if (-e "$path/$file")
		{
			# check if the md5 sum of the file matches the md5 of the file they want to sync
			open(FILE, "<$path/$file") || return "Can't read $path/$file: $!";
			if (md5_hex(<FILE>) eq $md5)
			{
				close(FILE);
				return 1;
			}
			close(FILE);
			return 0;
		}
		else
		{
			return 0;
		}
	}
}


__END__
]]></code>
</methoddef>
